@@ -24,7 +24,6 @@ package GNUpod::FooBar;
 
 use strict;
 use Digest::MD5;
-use GNUpod::iTunesDB;
 
 #####################################################################
 # Get paths / files
@@ -46,9 +45,12 @@ if(-d $opth->{mount}) {
   $rr->{itunesdb}       = $opth->{mount}."/iPod_Control/iTunes/iTunesDB";
   $rr->{playcounts}     = "$rr->{mountpoint}/iPod_Control/iTunes/Play Counts";
   $rr->{itunesdb_md5}   = "$rr->{etc}/.itunesdb_md5";
+  $rr->{onthego_invalid}  = "$rr->{etc}/.onthego_invalid";
   $rr->{onthego}        = "$rr->{mountpoint}/iPod_Control/iTunes/OTGPlaylistInfo";
   $rr->{status}         = undef;
 
+  _check_casesensitive($rr->{mountpoint}); #Check if somebody mounted the iPod caseSensitive
+      
  #Do an iTunesDB Sync if not disabled and needed
   do_itbsync($rr) if(!$opth->{_no_it_sync} && !$opth->{_no_sync} && _itb_needs_sync($rr));
  
@@ -62,6 +64,33 @@ elsif($opth->{mount}) {
  return $rr
 }
 
+#######################################################################
+# Check if someone mounted the iPod CaseSensitive
+sub _check_casesensitive {
+ my($target) = @_;
+ 
+ if(open(CSTEST,">$target/csTeSt")) {
+   my $inode_a = (stat("$target/csTeSt"))[1]; #Get inode of just-creaded file
+   my $inode_b = (stat("$target/CStEsT"))[1]; #Get inode of another file..
+   unlink("$target/csTeSt"); #Boom!
+  
+   if($inode_a != $inode_b) { #Whops, different inodes? -> case sensitive fs
+     #Nerv the user
+     warn "warning: $target seems to be mounted *CASE SENSITIVE*\n";
+     warn "         Mounting VFAT/HFS+ like this is a VERY BAD(tm) idea,\n";
+     warn "         strange things may happen... GNUpod may not work correctly!\n";
+     warn "         Please mount the Filesystem CASE *IN*SENSITIVE\n";
+     warn "         (use 'mount ... -o check=r' for VFAT)\n";
+     warn "         [Ignore this message if $target isn't a\n";
+     warn "          VFAT or HFS+ Filesystem ;) ]\n";
+   }
+  
+ }
+ else {
+   warn "warning: Could not write to $target, iPod mounted read-only? ($!)\n";
+ }
+}
+
 #######################################################################
 # Call tunes2pod
 sub do_itbsync {
@@ -160,7 +189,7 @@ sub _otg_needs_sync {
  my($rr) = @_;
 #warn "debug: otgsync need? (request from $$)\n";
  #OTG Sync needed
- return 1 if(GNUpod::iTunesDB::readOTG($rr->{onthego}));
+ return 1 if( (-e $rr->{onthego}) && (-s $rr->{onthego}) > 0 );
  
  if(-e $rr->{playcounts}) { #PlayCounts file exists..
   return 1;
@@ -171,6 +200,31 @@ sub _otg_needs_sync {
 }
 
 
+######################################################################
+# Check for broken onTheGo data (= GNUtunesDB <-> iTunesDB out of sync)
+sub _otgdata_broken {
+ my($rr) = @_;
+ return (-e $rr->{onthego_invalid});
+}
+
+######################################################################
+# Set otgdata synched
+sub setvalid_otgdata {
+ my($rr) = @_;
+ return undef unless -e $rr->{onthego_invalid};
+ unlink($rr->{onthego_invalid});
+}
+######################################################################
+# Set otgdata synched
+sub setINvalid_otgdata {
+ my($rr) = @_;
+ open(OTGINVALID, ">$rr->{onthego_invalid}") or die "Can't write $rr->{onthego_invalid}\n";
+  print OTGINVALID undef;
+ close(OTGINVALID);
+ return undef;
+}
+
+
 ######################################################################
 # Getmd5line
 sub getmd5line {
@@ -189,6 +243,7 @@ sub setsync {
  setsync_itunesdb($rr);
  setsync_playcounts($rr);
  setsync_otg($rr);
+ setvalid_otgdata($rr);
 }
 
 ######################################################################
@@ -234,6 +289,7 @@ my($rr) = @_;
 }
 
 
+
 ######################################################################
 # Get the MD5 sum of a file
 sub getmd5 {
@@ -246,4 +302,54 @@ sub getmd5 {
 }
 
 
+
+########################################################################
+# Parse configuration
+sub GetConfig {
+ my($getopts, $doset, $name) = @_;
+
+  my($topic,$val,$optarget);
+  
+  foreach my $filerc ( ("$ENV{HOME}/.gnupodrc", "$getopts->{mount}/iPod_Control/.gnupod/gnupodrc") ) {
+    open(RCFILE, $filerc) or next;
+     while (my $line = <RCFILE>) {
+      chomp($line);
+      next if !$line or $line =~ /^#/;
+      
+      #Ok, line is not a comment and has some content, read it..
+      unless(($topic,$val) = $line =~ /^(\S+)\s*=\s*(.+)$/) {
+       warn "warning: Invalid line '$line' found in $filerc\n";
+       next;
+      }
+      
+      #We matched and got $topic + $val, check it $topic has a
+      #specific target (like 'mktunes.volume')
+      if($topic =~ /^([^.]+)\.(.+)/) {
+       $optarget = $1;
+       $topic    = $2;
+      }
+      else { #No target found
+       $optarget = undef;
+      }
+
+  #    warn "### PARSE($line): *$topic* -> *$val*\n";
+  #    warn "### $topic with target $optarget\n";
+      
+         if ($optarget&&$name&&$name ne $optarget) { next}
+      elsif ($getopts->{$topic})      { next } #this is a dup 
+      elsif ($doset->{$topic} eq "s") { $getopts->{$topic} = $val }
+      elsif ($doset->{$topic} eq "i") { $getopts->{$topic} = int($val) }
+      elsif ($doset->{$topic} eq "b") { $getopts->{$topic} = 1 if($val && $val ne "no") }
+     }
+     close(RCFILE);
+ #    warn "** Parser finished $filerc\n";
+  }
+  
+ # foreach(keys(%$getopts)) {
+ #  warn "CONF: $_ - $getopts->{$_}\n";
+ # }
+  
+  
+  return 1;
+}
 1;
