@@ -1,5 +1,5 @@
 package GNUpod::XMLhelper;
-#  Copyright (C) 2002-2003 Adrian Ulrich <pab at blinkenlights.ch>
+#  Copyright (C) 2002-2004 Adrian Ulrich <pab at blinkenlights.ch>
 #  Part of the gnupod-tools collection
 #
 #  URL: http://www.gnu.org/software/gnupod/
@@ -27,7 +27,7 @@ use XML::Parser;
 use Unicode::String;
 use GNUpod::FooBar;
 
-## Release 20040531
+## Release 20040807
 
 my $cpn = undef; #Current PlaylistName
 my @idpub = ();
@@ -53,10 +53,11 @@ sub getpath {
    $path = $filename;
  }
  else { #Default action.. new filename to create 
+
    my $test_extension = $opts->{extension} || $opts->{format}; #Test extension
    my $name = (split(/\//, $filename))[-1];                    #Name
    my $i = 0;                                                  #Count
-   $name =~ tr/a-zA-Z0-9\./_/c;                                #CleanName
+   $name =~ tr/a-zA-Z0-9\./_/c;                                #CleanName for dumb Filesystems
 
    #Hups, current filename has a wrong extension...
    if($opts->{format} && $test_extension && ($name !~ /\.($test_extension)$/i)) {
@@ -65,19 +66,32 @@ sub getpath {
      $name =~ s/\.?$ext$/.$opts->{format}/;  #Replace current extension with newone
    }
  
-   #Search a place for the MP3 file
-   while($path = sprintf("$mountp/iPod_Control/Music/F%02d/%d_$name", int(rand(40)), $i++)) {
-     if( !(-e $path) && open(TESTFILE,">",$path) ) {
+   #### #Search a place for the MP3 file ####
+   # 1. Glob to find all dirs
+   ## We don't cache this, we do a glob for each new file..
+   ## Shouldn't be a waste if time.. We belive in the Cacheing of the OS ;)
+   my @aviable_targets = glob("$mountp/iPod_Control/Music/*");
+
+   # 2. Paranoia check..
+   unless(@aviable_targets) {
+     warn "No folders found, did you run gnupod_INIT.pl ?\n";
+     return undef;
+   }
+
+   # 3. Search
+   while($path = sprintf($aviable_targets[int(rand(@aviable_targets))]."/%d_$name",$i++)) {
+     if( !(-e $path) && open(TESTFILE,">",$path) ) { #This is false if we would write to a globed file :)
        close(TESTFILE);
        unlink($path); #Maybe it's a dup.. we don't create empty files
        last;
      }
      elsif($i > 2000) { #nuff!
-       warn "getpath(): Could not write $name anywhere, panic! [$!]\n";
+       warn "getpath(): Could not write $name anywhere, panic! [$path, $!]\n";
        return undef;
      }
    }
- }
+   
+ } ## End default action
  
 #Remove mountpoint from $path
 my $ipath = $path;
@@ -141,7 +155,9 @@ sub mkfile {
    else                { $r .= "/>"}
  }
   
-  if($magic->{return}) { return $r; }
+  if($magic->{return}) { 
+   return $r; 
+  }
   elsif($magic->{plname}) { #Create a playlist item
    push(@{$XDAT->{playlists}->{data}->{$magic->{plname}}}, $r);
   }
@@ -151,6 +167,7 @@ sub mkfile {
   else { #No playlist item? has to be a file
    push(@{$XDAT->{files}}, $r);
   }
+  return $xid; #Return current XID
 }
 
 ##############################################################
@@ -164,10 +181,11 @@ sub addpl {
    return;
  }
 
- push(@plorder, $name);
  my %rh = ();
     %rh = %{$opt} if ref($opt) eq "HASH"; #Copy if we got data 
- $rh{name} = $name; #Force the name
+ $rh{name} = $name;            #Force the name
+ $rh{plid} = int(rand(99999)); #We create our own id
+ push(@plorder, {name=>$name,plid=>$rh{plid}});
  
  $XDAT->{playlists}->{pref}->{$name} = \%rh;
 }
@@ -183,21 +201,31 @@ sub addspl {
    return;
  }
 
- push(@plorder, $name);
+
  my %rh = ();
     %rh = %{$opt} if ref($opt) eq "HASH"; #Copy if we got data 
- $rh{name} = $name; #Force the name
- 
+ $rh{name} = $name;              #Force the name
+ $rh{plid} ||= int(rand(99999)); #We create our own id
+ push(@plorder, {name=>$name,plid=>$rh{plid}});
+  
  $XDAT->{spls}->{pref}->{$name} = \%rh;
 }
 
 
 ##############################################################
-#Get all playlists
-sub getpl_names {
+#Get all playlists {name, plid}
+sub getpl_attribs {
  return @plorder;
 }
 
+##############################################################
+#Get Playlist content
+sub getpl_content {
+ my($plname) = @_;
+ return @{$XDAT->{playlists}->{data}->{$plname}} if ref($XDAT->{playlists}->{data}->{$plname}) eq "ARRAY";
+ return (); #Dummy fallback
+}
+
 ##############################################################
 # Get SPL Prefs
 sub get_splpref {
@@ -217,7 +245,7 @@ sub eventer {
  return undef unless $href->{Context}[0] eq "gnuPod";
   if($href->{Context}[1] eq "files") {
     #add(s)pl() call done before we got all files? that's bad!
-    warn "** XMLhelper: Found <file ../> item *after* a <playlist ..>, that's bad\n" if getpl_names();
+    warn "** XMLhelper: Found <file ../> item *after* a <playlist ..>, that's bad\n" if getpl_attribs();
     my $xh = mkh($el,@it);         #Create a hash
     @idpub[$xh->{file}->{id}] = 1; #Promote ID
     main::newfile($xh);            #call sub
@@ -290,19 +318,21 @@ sub writexml {
 #End file part
 
 #Print all playlists
- foreach(@plorder) {
-  if(my $shr = get_splpref($_)) { #xmlheader present
+ foreach(getpl_attribs()) {
+  my $current_plname = $_->{name};
+  
+  if(my $shr = get_splpref($current_plname)) { #xmlheader present
       print OUT "\n ".mkfile({smartplaylist=>$shr}, {return=>1,noend=>1})."\n";
        ### items
-        foreach my $sahr (@{$XDAT->{spls}->{data}->{$_}}) {
+        foreach my $sahr (@{$XDAT->{spls}->{data}->{$current_plname}}) {
          print OUT "   $sahr\n";
         }
        ###
       print OUT " </smartplaylist>\n";
   }
-  elsif(my $phr = get_plpref($_)) { #plprefs found..
+  elsif(my $phr = get_plpref($current_plname)) { #plprefs found..
       print OUT "\n ".mkfile({playlist=>$phr}, {return=>1,noend=>1})."\n";
-       foreach(@{$XDAT->{playlists}->{data}->{$_}}) {
+       foreach(@{$XDAT->{playlists}->{data}->{$current_plname}}) {
         print OUT "   $_\n";
        }
       print OUT " </playlist>\n";
