@@ -92,7 +92,7 @@ $SPLDEF{field}{25} = "rating";
 $SPLDEF{field}{31} = "compilation";
 $SPLDEF{field}{35} = "bpm";
 $SPLDEF{field}{39} = "GROUP";
-$SPLDEF{field}{40} = "PLAYLIST";
+$SPLDEF{field}{40} = "playlist";
 
 
 #Checkrule (COMPLETE)
@@ -245,7 +245,8 @@ sub mk_mhit {
     $ret .= pack("h8", _itop($file_hash{cds}));               #number of cds
     $ret .= pack("H8");                                       #hardcoded space ?
     $ret .= pack("h8", _itop($file_hash{addtime}));           #File added @
-    $ret .= pack("H16");
+    $ret .= pack("h8", _itop($file_hash{bookmark}));          #QTFile Bookmark
+    $ret .= pack("H8");
     $ret .= pack("H12");                                       #??
     $ret .= pack("h4", _itop($file_hash{bpm},0xffff));         #BPM
 #Fixme: this was wrong.. so i removed it now..
@@ -572,6 +573,7 @@ my $appnd = mk_mhod({stype=>"title", string=>$hr->{name}}).__dummy_listview();
 ##have to adjust the childs here
 my $cmh = 2+$hr->{mhods};
 
+
 my $ret .= "mhyp";
    $ret .= pack("h8", _itop(108)); #type
    $ret .= pack("h8", _itop($hr->{size}+108+(length($appnd))));          #size
@@ -579,7 +581,7 @@ my $ret .= "mhyp";
    $ret .= pack("h8", _itop($hr->{files}));   #songs in pl
    $ret .= pack("h8", _itop($hr->{type}));    # 1 = main .. 0=not main
    $ret .= pack("H8", "00"); 			      #?
-   $ret .= pack("H8", "00");                  #?
+   $ret .= pack("h8", _itop($hr->{plid}));    #Playlist ID
    $ret .= pack("H8", "00");                  #?
    $ret .= pack("H144", "00");       		  #dummy space
 
@@ -957,11 +959,13 @@ sub get_pl {
  my @pldata = ();
  
   if(get_string($pos, 4) eq "mhyp") { #Ok, its an mhyp
-      $ret_hash{type} = get_int($pos+20, 4); #Is it a main playlist?
-   my $scount         = get_int($pos+16, 4); #How many songs should we expect?
    my $header_len     = get_int($pos+4, 4);  #Size of the header
-   my $mhyp_len       = get_int($pos+8, 4);   #Size of mhyp
-   my $mhods          = get_int($pos+12,4); #How many mhods we have here
+   my $mhyp_len       = get_int($pos+8, 4);  #Size of mhyp
+   my $mhods          = get_int($pos+12,4);  #How many mhods we have here
+   my $scount         = get_int($pos+16, 4); #How many songs should we expect?
+      $ret_hash{type} = get_int($pos+20, 4); #Is it a main playlist?
+      $ret_hash{plid}  = get_int($pos+28,4);  #UID if the playlist..
+ 
 #Its a MPL, do a fast skip  --> We don't parse the mpl, because we know the content anyway
 if($ret_hash{type}) {
  return ($pos+$mhyp_len, {type=>1}) 
@@ -1019,11 +1023,10 @@ if(get_string($sum, 4) eq "mhit") { #Ok, its a mhit
 my %ret     = ();
 #Infos stored in mhit
 $ret{id}         = get_int($sum+16,4);
+$ret{rating}     = int((get_int($sum+28,4)-256)/oct('0x14000000')) * 20;
 $ret{changetime} = get_int($sum+32,4);
 $ret{filesize}   = get_int($sum+36,4);
 $ret{time}       = get_int($sum+40,4);
-$ret{cdnum}      = get_int($sum+92,4);
-$ret{cds}        = get_int($sum+96,4);
 $ret{songnum}    = get_int($sum+44,4);
 $ret{songs}      = get_int($sum+48,4);
 $ret{year}       = get_int($sum+52,4);
@@ -1035,8 +1038,10 @@ $ret{stoptime}   = get_int($sum+72,4);
 $ret{soundcheck} = get_int($sum+76,4);
 $ret{playcount}  = get_int($sum+80,4); #84 has also something to do with playcounts. (Like rating + prerating?)
 $ret{lastplay}   = get_int($sum+88,4);
-$ret{rating}     = int((get_int($sum+28,4)-256)/oct('0x14000000')) * 20;
+$ret{cdnum}      = get_int($sum+92,4);
+$ret{cds}        = get_int($sum+96,4);
 $ret{addtime}    = get_int($sum+104,4);
+$ret{bookmark}   = get_int($sum+108,4);
 $ret{bpm} = get_int($sum+122,2);
 
 #Fixme: prerating is invalid.. rerere..
@@ -1131,6 +1136,7 @@ sub readPLC {
  my %pcrh = ();
  my $rating   = 0;
  my $playc    = 0;
+ my $bookmark = 0;
  my $lastply  = 0;
  my $chunknum = 0;
 
@@ -1149,8 +1155,12 @@ sub readPLC {
   if (read(RATING,$buff,4) != 4) { _itBUG("Read failed at $offset while reading LASTPLAY ($chunks/$chunksize)"); last; }
   $lastply = GNUpod::FooBar::shx2int($buff);
   
-  #8 is always zero atm?!
-
+  if($chunksize >= 12) {
+   seek(RATING, $offset+8,0);
+   if(read(RATING, $buff,4) != 4) { _itBUG("Read failed at $offset while reading BOOKMARK ($chunks/$chunksize)"); last;}
+   $bookmark = GNUpod::FooBar::shx2int($buff);
+  }
+  
   if($chunksize >= 16) { #12+4 - v2 firmware? 
    seek(RATING, $offset+12, 0);
    if (read(RATING, $buff,4) != 4) { _itBUG("Read failed at $offset while reading RATING ($chunks/$chunksize)"); last; }
@@ -1159,9 +1169,10 @@ sub readPLC {
   
 #print " aka $chunknum] \n";
 
-  $pcrh{playcount}{$chunknum} = $playc   if $playc;
-  $pcrh{rating}{$chunknum}    = $rating  if $rating;
-  $pcrh{lastplay}{$chunknum}  = $lastply if $lastply;
+  $pcrh{playcount}{$chunknum} = $playc    if $playc;
+  $pcrh{rating}{$chunknum}    = $rating   if $rating;
+  $pcrh{lastplay}{$chunknum}  = $lastply  if $lastply;
+  $pcrh{bookmark}{$chunknum}  = $bookmark if $bookmark;
   $offset += $chunksize; #Nex to go!
  }
 
