@@ -30,7 +30,7 @@ use Getopt::Long;
 use File::Copy;
 
 use constant MACTIME => 2082931200; #Mac EPOCH offset
-use vars qw(%opts %dupdb $int_count);
+use vars qw(%opts %dupdb_normal %dupdb_lazy $int_count);
 
 print "gnupod_addsong.pl Version ###__VERSION__### (C) Adrian Ulrich\n";
 
@@ -38,11 +38,13 @@ $int_count = 3; #The user has to send INT (Ctrl+C) x times until we stop
 
 $opts{mount} = $ENV{IPOD_MOUNTPOINT};
 #Don't add xml and itunes opts.. we *NEED* the mount opt to be set..
-GetOptions(\%opts, "version", "help|h", "mount|m=s", "decode", "restore|r", "duplicate|d", "disable-v2", "disable-v1",
-                   "set-artist=s", "set-album=s", "set-genre=s", "set-rating=i", "set-playcount=i");
-GNUpod::FooBar::GetConfig(\%opts, {'decode'=>'b', mount=>'s', duplicate=>'b', 'disable-v1'=>'b', 'disable-v2'=>'b'},
+GetOptions(\%opts, "version", "help|h", "mount|m=s", "decode=s", "restore|r", "duplicate|d", "disable-v2", "disable-v1",
+                   "set-artist=s", "set-album=s", "set-genre=s", "set-rating=i", "set-playcount=i", "playlist|p=s");
+GNUpod::FooBar::GetConfig(\%opts, {'decode'=>'s', mount=>'s', duplicate=>'b', 'disable-v1'=>'b', 'disable-v2'=>'b'},
                           "gnupod_addsong");
 
+
+usage("\n--decode takes 'pcm' 'mp3' 'aac' 'aacbm' -> '--decode=mp3'\n") if $opts{decode} && $opts{decode} !~ /^(mp3|aac|aacbm|pcm|crashme)$/;
 usage() if $opts{help};
 version() if $opts{version};
 
@@ -51,8 +53,9 @@ if($opts{restore}) {
  print "If you use --restore, you'll *lose* your playlists\n";
  print " Hit ENTER to continue or CTRL+C to abort\n\n";
  <STDIN>;
- delete($opts{decode}); #We don't decode anything
- $opts{duplicate} = 1;  #Don't skip dups on restore
+ delete($opts{decode});    #We don't decode anything
+ $opts{duplicate} = 1;     #Don't skip dups on restore
+ $opts{decode}    = undef; #Do not encode, only native files are on an iPod
  startup(glob("$opts{mount}/iPod_Control/Music/*/*"));
 }
 elsif($ARGV[0] eq "-" && @ARGV == 1) {
@@ -82,9 +85,14 @@ sub startup {
  usage($con->{status}."\n") if $con->{status} || !@files;
 
  unless($opts{restore}) { #We parse the old file, if we are NOT restoring the iPod
-  GNUpod::XMLhelper::doxml($con->{xml}) or usage("Failed to parse $con->{xml}\n");
+  GNUpod::XMLhelper::doxml($con->{xml}) or usage("Failed to parse $con->{xml}, did you run gnupod_INIT.pl?\n");
  }
 
+ if ($opts{playlist}) { #Create this playlist
+  print "> Adding songs to Playlist '$opts{playlist}'\n";
+  GNUpod::XMLhelper::addpl($opts{playlist}); #Fixme: this may printout a warning..
+ } 
+
  my $addcount = 0;
 
  #We are ready to copy each file..
@@ -95,17 +103,18 @@ sub startup {
     next if -d $file;
     
     #Get the filetype
-    my ($fh,$media_h,$converted) =  GNUpod::FileMagic::wtf_is($file, {noIDv1=>$opts{'disable-v1'}, 
+    my ($fh,$media_h,$converter) =  GNUpod::FileMagic::wtf_is($file, {noIDv1=>$opts{'disable-v1'}, 
                                                                       noIDv2=>$opts{'disable-v2'},
-								                                      decode=>$opts{'decode'}});
+								                                      decode=>$opts{'decode'}},$con);
+
     unless($fh) {
-     warn "* [****] Skipping '$file', unknown file type\n";
+    warn "* [****] Skipping '$file', unknown file type\n";
      next;
     }
     
-   my $wtf_ftyp = $media_h->{ftyp};
-   my $wtf_frmt = $media_h->{format};
-   my $wtf_ext  = $media_h->{extension};
+   my $wtf_ftyp = $media_h->{ftyp};      #'codec' .. maybe ALAC
+   my $wtf_frmt = $media_h->{format};    #container ..maybe M4A
+   my $wtf_ext  = $media_h->{extension}; #Possible extensions (regexp!)
    
    #wtf_is found a filetype, override data if needed
    $fh->{artist}    = $opts{'set-artist'}    if $opts{'set-artist'};
@@ -113,22 +122,54 @@ sub startup {
    $fh->{genre}     = $opts{'set-genre'}     if $opts{'set-genre'};
    $fh->{rating}    = $opts{'set-rating'}    if $opts{'set-rating'};
    $fh->{playcount} = $opts{'set-playcount'} if $opts{'set-playcount'};
-   
    #Set the addtime to unixtime(now)+MACTIME (the iPod uses mactime)
    $fh->{addtime} = time()+MACTIME;
+
+
+   #Check for duplicates
+   if(!$opts{duplicate} && (my $dup = checkdup($fh,$converter))) {
+    print "! [!!!] '$file' is a duplicate of song $dup, skipping file\n";
+    create_playlist_now($opts{playlist}, $dup); #We also add duplicates to a playlist..
+    next;
+   }
+
+   ## Ok, file is not a duplicate
+
+   
+   if($converter) {
+    print "> Converting '$file' from $wtf_ftyp into $opts{decode}, please wait...\n";
+    my $path_of_converted_file = GNUpod::FileMagic::kick_convert($converter,$file, uc($opts{decode}), $con);
+    unless($path_of_converted_file) {
+     print "! [!!!] Could not convert $file\n";
+     next;
+    }
+
+    #Ok, we got a converted file, fillout the gaps
+    my($conv_fh, $conv_media_h) = GNUpod::FileMagic::wtf_is($path_of_converted_file, undef, $con);
+    
+    unless($conv_fh) {
+     warn "* [***] Internal problem: $converter did not produce valid data.\n";
+     warn "* [***] Something is wrong with $path_of_converted_file (file not deleted, debug it! :) )\n";
+     next;
+    }
+    
+    #We didn't know things like 'filesize' before...
+    $fh->{time}     = $conv_fh->{time};
+    $fh->{bitrate}  = $conv_fh->{bitrate};
+    $fh->{srate}    = $conv_fh->{srate};        
+    $fh->{filesize} = $conv_fh->{filesize};   
+    $wtf_frmt = $conv_media_h->{format};    #Set the new format (-> container)
+    $wtf_ext  = $conv_media_h->{extension}; #Set the new possible extension
+    #BUT KEEP ftyp! (= codec)
+    
+    $file = $path_of_converted_file; #Point $file to new file
+   }
+
+
    #Get a path
    (${$fh}{path}, my $target) = GNUpod::XMLhelper::getpath($opts{mount}, $file, 
                                                            {format=>$wtf_frmt, extension=>$wtf_ext, keepfile=>$opts{restore}});
-   #Check for duplicates
-   if(!$opts{duplicate} && (my $dup = checkdup($fh))) {
-    print "! [!!!] '$file' is a duplicate of song $dup, skipping file\n";
-    unlink($converted) if $converted; #Unlink file, if we converted it.. (tmp)
-    next;
-   }
-   
-   
-   #ReSet filename if we did a convert
-   $file = $converted if $converted;
+
    
    if(!defined($target)) {
     warn "*** FATAL *** Skipping '$file' , no target found!\n";
@@ -137,24 +178,19 @@ sub startup {
      printf("+ [%-4s][%3d] %-32s | %-32s | %-24s\n",
 	    uc($wtf_ftyp),1+$addcount, $fh->{title}, $fh->{album},$fh->{artist});
      
-	 my $fmh;
-     $fmh->{file} = $fh;
-     GNUpod::XMLhelper::mkfile($fmh,{addid=>1}); #Try to add an id
+     my $id = GNUpod::XMLhelper::mkfile({file=>$fh},{addid=>1}); #Try to add an id
+     create_playlist_now($opts{playlist}, $id);
      $addcount++; #Inc. addcount
    }
    else { #We failed..
      warn "*** FATAL *** Could not copy '$file' to '$target': $!\n";
    }
-   
-   #Now we unlink leftover converted files even if we couldn't
-   #copy the file to the iPod
-   unlink($converted) if $converted;
-   
+   unlink($file) if $converter; #File is in $tmp if $converter is set...
  }
 
  
  
- if($addcount) { #We have to modify the xmldoc
+ if($opts{playlist} || $addcount) { #We have to modify the xmldoc
   print "> Writing new XML File, added $addcount file(s)\n";
   GNUpod::XMLhelper::writexml($con);
  }
@@ -164,14 +200,34 @@ sub startup {
 
 
 
+#############################################################
+# Add item to playlist
+sub create_playlist_now {
+ my($plname, $id) = @_;
 
-
+ if($plname && $id >= 0) {
+   #Broken-by-design: We don't have a ID-Pool for playlists..
+   #-> Create a fake_entry
+   my $fake_entry = GNUpod::XMLhelper::mkfile({ add => { id => $id } }, { return=>1 });
+   my $found = 0;
+   foreach(GNUpod::XMLhelper::getpl_content($plname)) {
+     if($_ eq $fake_entry) {
+       $found++; last;
+     }
+   }
+   GNUpod::XMLhelper::mkfile({ add => { id => $id } },{"plname"=>$plname}) unless $found;
+ }
+}
 
 
 
 ## XML Handlers ##
 sub newfile {
- $dupdb{lc($_[0]->{file}->{title})."/$_[0]->{file}->{bitrate}/$_[0]->{file}->{time}/$_[0]->{file}->{filesize}"}= $_[0]->{file}->{id}||-1;
+ $dupdb_normal{lc($_[0]->{file}->{title})."/$_[0]->{file}->{bitrate}/$_[0]->{file}->{time}/$_[0]->{file}->{filesize}"}= $_[0]->{file}->{id}||-1;
+
+#This is worse than _normal, but the only way to detect dups *before* re-encoding...
+ $dupdb_lazy{lc($_[0]->{file}->{title})."/".lc($_[0]->{file}->{album})."/".lc($_[0]->{file}->{artist})}= $_[0]->{file}->{id}||-1;
+
  GNUpod::XMLhelper::mkfile($_[0],{addid=>1});
 }
 
@@ -184,8 +240,12 @@ sub newpl {
 ###############################################################
 # Check if the file is a duplicate
 sub checkdup {
- my($fh) = @_;
- return $dupdb{lc($fh->{title})."/$fh->{bitrate}/$fh->{time}/$fh->{filesize}"};
+ my($fh, $from_lazy) = @_;
+ 
+ return  $dupdb_lazy{lc($_[0]->{title})."/".lc($_[0]->{album})."/".lc($_[0]->{artist})}
+   if $from_lazy;
+   
+ return $dupdb_normal{lc($fh->{title})."/$fh->{bitrate}/$fh->{time}/$fh->{filesize}"};
 }
 
 
@@ -210,19 +270,20 @@ die << "EOF";
 $rtxt
 Usage: gnupod_addsong.pl [-h] [-m directory] File1 File2 ...
 
-   -h, --help               display this help and exit
-       --version            output version information and exit
-   -m, --mount=directory    iPod mountpoint, default is \$IPOD_MOUNTPOINT
-   -r, --restore            Restore the iPod (create a new GNUtunesDB from scratch)
-   -d, --duplicate          Allow duplicate files
-       --disable-v1         Do not read ID3v1 Tags (MP3 Only)
-       --disable-v2         Do not read ID3v2 Tags (MP3 Only)
-       --decode             Convert FLAC Files to WAVE 'onthefly'
-       --set-artist=string  Set Artist (Override ID3 Tag)
-       --set-album=string   Set Album  (Override ID3 Tag)
-       --set-genre=string   Set Genre  (Override ID3 Tag)
-       --set-rating=int     Set Rating
-       --set-playcount=int  Set Playcount
+   -h, --help                      display this help and exit
+       --version                   output version information and exit
+   -m, --mount=directory           iPod mountpoint, default is \$IPOD_MOUNTPOINT
+   -r, --restore                   Restore the iPod (create a new GNUtunesDB from scratch)
+   -d, --duplicate                 Allow duplicate files
+   -p, --playlist=string           Add songs to this playlist
+       --disable-v1                Do not read ID3v1 Tags (MP3 Only)
+       --disable-v2                Do not read ID3v2 Tags (MP3 Only)
+       --decode=pcm|mp3|aac|aacbm  Convert FLAC Files to WAVE/MP3 or AAC 'on-the-fly'
+       --set-artist=string         Set Artist (Override ID3 Tag)
+       --set-album=string          Set Album  (Override ID3 Tag)
+       --set-genre=string          Set Genre  (Override ID3 Tag)
+       --set-rating=int            Set Rating
+       --set-playcount=int         Set Playcount
 
 Report bugs to <bug-gnupod\@nongnu.org>
 EOF
