@@ -1,47 +1,44 @@
 ###__PERLBIN__###
-#  Copyright (C) 2002-2007 Adrian Ulrich <pab at blinkenlights.ch>
+#  Copyright (C) 2002-2004 Adrian Ulrich <pab at blinkenlights.ch>
 #  Part of the gnupod-tools collection
 #
 #  URL: http://www.gnu.org/software/gnupod/
 #
-#    GNUpod is free software; you can redistribute it and/or modify
+#    This program is free software; you can redistribute it and/or modify
 #    it under the terms of the GNU General Public License as published by
-#    the Free Software Foundation; either version 3 of the License, or
+#    the Free Software Foundation; either version 2 of the License, or
 #    (at your option) any later version.
 #
-#    GNUpod is distributed in the hope that it will be useful,
+#    This program is distributed in the hope that it will be useful,
 #    but WITHOUT ANY WARRANTY; without even the implied warranty of
 #    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 #    GNU General Public License for more details.
 #
 #    You should have received a copy of the GNU General Public License
-#    along with this program.  If not, see <http://www.gnu.org/licenses/>.#
+#    along with this program; if not, write to the Free Software
+#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 # iTunes and iPod are trademarks of Apple
 #
 # This product is not supported/written/published by Apple!
 
 use strict;
-use GNUpod::FooBar;
-use GNUpod::FileMagic;
+
+
 
 my $file  = $ARGV[0] or exit(1);
 my $gimme = $ARGV[1];
-my $quality = $ARGV[2];
 
 
-if(!(-r $file)) {
- warn "$file is not readable!\n";
- exit(1);
-}
-elsif($gimme eq "GET_META") {
+if($gimme eq "GET_META") {
  my $ftag = undef;
  ## This is a UGLY trick to cheat perl!
  ## 1. Create a string
  
  my $ogg_pmod = undef;
  
- foreach my $ogg_pmod ('Ogg::Vorbis::Header::PurePerl','Ogg::Vorbis::Header') {
+ for( ('Ogg::Vorbis::Header::PurePerl','Ogg::Vorbis::Header') ) {
+     $ogg_pmod = $_;
   my $nocompile = "use $ogg_pmod; \$ftag = $ogg_pmod->new(\$file);";
   eval $nocompile; #2. eval it!
   last unless $@;
@@ -59,14 +56,12 @@ print "_ARTIST:".($ftag->comment('artist'))[0]."\n";
 print "_ALBUM:".($ftag->comment('album'))[0]."\n";
 print "_TITLE:".($ftag->comment('title'))[0]."\n";
 print "_GENRE:".($ftag->comment('genre'))[0]."\n";
-print "_TRACKNUM:".( ($ftag->comment('tracknum'))[0] |
-                     ($ftag->comment('tracknumber'))[0] )."\n";
+print "_TRACKNUM:".($ftag->comment('tracknum'))[0]."\n";
 print "_COMMENT:".($ftag->comment('comment'))[0]."\n";
-print "_MEDIATYPE:".(GNUpod::FileMagic::MEDIATYPE_AUDIO)."\n";
-print "FORMAT:OGG\n";
+print "FORMAT: OGG\n";
 }
 elsif($gimme eq "GET_PCM") {
-  my $tmpout = GNUpod::FooBar::get_u_path("/tmp/gnupod_pcm", "wav");
+  my $tmpout = get_u_path("/tmp/gnupod_pcm", "wav");
 
   my $status = system("oggdec", "--quiet", "-o", $tmpout, $file);
   
@@ -81,9 +76,9 @@ elsif($gimme eq "GET_PCM") {
 elsif($gimme eq "GET_MP3") {
   #Open a secure flac pipe and open anotherone for lame
   #On errors, we'll get a BrokenPipe to stout
-  my $tmpout = GNUpod::FooBar::get_u_path("/tmp/gnupod_mp3", "mp3");
+  my $tmpout = get_u_path("/tmp/gnupod_mp3", "mp3");
   open(OGGOUT, "-|") or exec("oggdec", "--quiet", "-o", "-", $file) or die "Could not exec oggdec: $!\n";
-  open(LAMEIN , "|-") or exec("lame", "-V", $quality, "--silent", "-", $tmpout) or die "Could not exec lame: $!\n";
+  open(LAMEIN , "|-") or exec("lame", "--silent", "--preset","extreme", "-", $tmpout) or die "Could not exec lame: $!\n";
    while(<OGGOUT>) {
     print LAMEIN $_;
    }
@@ -93,11 +88,10 @@ elsif($gimme eq "GET_MP3") {
 }
 elsif($gimme eq "GET_AAC" or $gimme eq "GET_AACBM") {
  #Yeah! FAAC is broken and can't write to stdout..
-  my $tmpout = GNUpod::FooBar::get_u_path("/tmp/gnupod_faac", "m4a");
-     $tmpout = GNUpod::FooBar::get_u_path("/tmp/gnupod_faac", "m4b") if $gimme eq "GET_AACBM";
-  $quality = 140 - ($quality*10);
+  my $tmpout = get_u_path("/tmp/gnupod_faac", "m4a");
+     $tmpout = get_u_path("/tmp/gnupod_faac", "m4b") if $gimme eq "GET_AACBM";
   open(OGGOUT, "-|") or exec("oggdec", "--quiet", "-o", "-", $file) or die "Could not exec oggdec: $!\n";
-  open(FAACIN , "|-") or exec("faac", "-w", "-q", $quality, "-o", $tmpout, "-") or die "Could not exec faac: $!\n";
+  open(FAACIN , "|-") or exec("faac", "-w", "-q", "120", "-o", $tmpout, "-") or die "Could not exec faac: $!\n";
    while(<OGGOUT>) { #Feed faac
     print FAACIN $_;
    }
@@ -113,3 +107,14 @@ else {
 }
 
 exit(0);
+
+#############################################
+# Get Unique path
+sub get_u_path {
+ my($prefix, $ext) = @_;
+ my $dst = undef;
+ while($dst = sprintf("%s_%d_%d.$ext",$prefix, time(), int(rand(99999)))) {
+  last unless -e $dst;
+ }
+ return $dst;
+}
