@@ -99,8 +99,9 @@ sub mkotg {
  
  #Get all old playlists and create a new name
  my $otggen = 1;
- foreach(GNUpod::XMLhelper::getpl_names()) {
-   if($_ =~ /^On-The-Go (\d+)/) {
+ foreach(GNUpod::XMLhelper::getpl_attribs()) {
+   my $plname = $_->{name};
+   if($plname =~ /^On-The-Go (\d+)/) {
     $otggen = ($1+1) if $otggen<=$1;
    }
  }
@@ -133,12 +134,10 @@ sub newfile {
   #Adjust rating
   $el->{file}->{rating}    = $plcref->{rating}{int(@keeper)-1};
   $el->{file}->{playcount} += $plcref->{playcount}{int(@keeper)-1};
+  $el->{file}->{bookmark}  = $plcref->{bookmark}{int(@keeper)-1};
   
   if($plcref->{lastplay}{int(@keeper)-1}) {
     $el->{file}->{lastplay}  = $plcref->{lastplay}{int(@keeper)-1};
-    
-#    print "*" x (int($el->{file}->{rating}/20));
-#    print " $el->{file}->{id} has a lastplay of $el->{file}->{lastplay} !\n";
   }
  
  }
