@@ -52,7 +52,7 @@ sub go {
 
  pview(undef,1);
  
- GNUpod::XMLhelper::doxml($con->{xml}) or usage("Failed to parse $con->{xml}\n");
+ GNUpod::XMLhelper::doxml($con->{xml}) or usage("Failed to parse $con->{xml}, did you run gnupod_INIT.pl?\n");
  #XML::Parser finished, write new file
  GNUpod::XMLhelper::writexml($con) if $opts{delete};
 
@@ -92,7 +92,7 @@ sub newpl {
  my ($el, $name, $plt) = @_;
  if($plt eq "pl" && ref($el->{add}) eq "HASH") { #Add action
   if(defined($el->{add}->{id}) && int(keys(%{$el->{add}})) == 1) { #Only id
-   return unless($keeplist[$el->{add}->{id}]); #ID not on keeplist. dropt it
+   return unless($keeplist[$el->{add}->{id}]); #ID not on keeplist. drop it
   }
  }
  elsif($plt eq "spl" && ref($el->{splcont}) eq "HASH") { #spl content
